package com.tzld.piaoquan.ad.engine.service.score.impl;

import com.tzld.piaoquan.ad.engine.commons.score.ScoreParam;
import com.tzld.piaoquan.ad.engine.service.score.RankService680;
import com.tzld.piaoquan.ad.engine.service.score.dto.AdDirectionScore;
import com.tzld.piaoquan.ad.engine.service.score.dto.AdPlatformCreativeDTO;
import com.tzld.piaoquan.ad.engine.service.score.param.RankRecommendRequestParam;
import com.tzld.piaoquan.recommend.feature.domain.ad.base.AdRankItem;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.*;
import java.util.stream.Collectors;

@Slf4j
@Service
public class TacticsAndLRModelScoreRankService {

    @Autowired
    private RankService680 rankService680;

    public List<AdRankItem> adItemRank(RankRecommendRequestParam requestParam, ScoreParam scoreParam) {

        // LR模型打分结果
        List<AdRankItem> result = rankService680.adItemRank(requestParam, scoreParam);

        Map<Long, AdDirectionScore> adDirectionScoreMap = requestParam.getAdIdList().stream()
                .collect(Collectors.toMap(AdPlatformCreativeDTO::getCreativeId, AdPlatformCreativeDTO::getAdDirectionScore));

        for (AdRankItem adRankItem : result) {

            this.calcDirectionScore(adRankItem, adDirectionScoreMap.get(adRankItem.getAdId()));

            double s1 = adRankItem.getAdDirectionScore();
            // 模型的打分结果已经乘CPA，此处不需要重复乘CPA
            double s2 = adRankItem.getScore();
            adRankItem.setScore(s1 * s2);

        }

        Collections.sort(result);
        return result;
    }

    private double calcDirectionScore(AdRankItem adRankItem, AdDirectionScore adDirectionScore) {
        if (Objects.isNull(adDirectionScore)) {
            adRankItem.getScoreMap().put("adDirectionScore", 1);
            adRankItem.setAdDirectionScore(1);
            return 1;
        }
        double exponent = Objects.nonNull(adDirectionScore.getExponent()) ? adDirectionScore.getExponent() : 1;
        Map<String, String> scoreDetail = adDirectionScore.getScoreDetail();

        double network = Double.parseDouble(scoreDetail.getOrDefault("network", "1"));
        double phoneModel = Double.parseDouble(scoreDetail.getOrDefault("phoneModel", "1"));
        double area = Double.parseDouble(scoreDetail.getOrDefault("area", "1"));
        double peoplePackage = Double.parseDouble(scoreDetail.getOrDefault("peoplePackage", "1"));
        double excludeConv = Double.parseDouble(scoreDetail.getOrDefault("excludeConv", "1"));
        double excludeClick = Double.parseDouble(scoreDetail.getOrDefault("excludeClick", "1"));
        double excludeView = Double.parseDouble(scoreDetail.getOrDefault("excludeView", "1"));

        double excludeMin = Math.min(excludeConv, Math.min(excludeClick, excludeView));

        double s1 = (network * phoneModel * area * peoplePackage * excludeMin);
        double s2 = Math.pow(s1, exponent);
        adRankItem.getScoreMap().put("adDirectionScore", s2);
        adRankItem.setAdDirectionScore(s2);

        Map<String, String> scoreDetailMap = new HashMap<>(scoreDetail);
        scoreDetailMap.put("exponent", String.valueOf(exponent));
        adRankItem.getMetaFeatureMap().put("adDirectionScoreDetail", scoreDetailMap);

        return s2;
    }

}
